//////////////////////////////////////////////////////////////////////////////
//  Compiler include header
//
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <memory.h>
#include "vconvw.h"


//////////////////////////////////////////////////////////////////////////////
// Voice File Convert Reserve Table
//
int _nMaxTbl3Bit[4] = { 0x399A, 0x3A9F, 0x4D14, 0x6607 } ;
int _nMaxTbl2Bit[4] = { 0x3800, 0x5600, 0x4D14, 0x6607 } ;

//////////////////////////////////////////////////////////////////////////////
// Reserved buffers for the ZyXEL ADPCM file header
//
BYTE _szZheadBuf1[16] = { 'Z','y','X','E','L',2,0,0,0,0,0,0,0,0,0,0 } ;

//////////////////////////////////////////////////////////////////////////////
// Reserved buffers for the .VOC file header
//
BYTE _szVheadBuf1[32] = { 'C','r','e','a','t','i','v','e',' ',\
			  'V','o','i','c','e',' ','F','i','l','e',\
			  0x1a,0x1a,0,0x0a,1,0x29,0x11,\
			  1,0x82,0x70,0,0x98,0 } ;

BYTE _szVheadBuf2[4]  = { 2, 0x80, 0x70, 0 } ;


////////////////////////////////////////////////////////////////////////////
//
//  _lseekf(
//

long _lseekf(
FILE *f,
long lOffset,
int nOrigin)
{
   fseek(f, lOffset, nOrigin);
   return ftell(f);
}



////////////////////////////////////////////////////////////////////////////
//	ToVoc()
//	Convert Adpcm (2 / 3 bit) data to VOC & return 8 bit uncompress data
//
// Copyright 1992, ZyXEL Communications Corporation
// Modified by Valen Liao Apr22 "95
//
WORD ToVoc(
int nVdata,
LPSTR lpzf)
{
	Adaptive((char)LOBYTE(nVdata), lpzf) ;
	return Voc8Bit(lpzf);
}



////////////////////////////////////////////////////////////////////////////
//	ToAdpcm()
//
//	Convert VOC to Adpcm (2 / 3 bit) data  & return 8 bit compress data
//	& write adpcm data to file
//
// Copyright 1992, ZyXEL Communications Corporation
// Modified by Valen Liao, Apr22 "95
//
int ToAdpcm(
int nEdata,
LPSTR lpzf,
LPSTR lpzfadpcm)
{
	WORD wRet;

	// Check for the waveform data and quantize this data
	// return wRet HIBYTE = bfPack flag
	//	       LOBYTE = cbDataBit
	wRet = Quantize8BitWave(0, nEdata, lpzf, lpzfadpcm);
	// Adaptive the Delta and Estimat Maximum
	Adaptive((char)LOBYTE(wRet), lpzf) ;

	return (int)HIBYTE(wRet);
}


// 
// BOOL FAR PASCAL WriteConvertFile(hAd, lpAdBuf, nAdBufptr)
//
// Copyright 1992, ZyXEL Communications Corporation
// Modified By Valen Liao, Apr22 "95
// 

int WriteConvertFile(
FILE *f,
char * npBuf,
int nBufptr)
{
    fwrite(npBuf, sizeof(char), nBufptr, f);

//  int nWByte = 0;
//  int nWTotalByte = 0;
//  while (nWTotalByte < nAdBufptr)
//  {
//     if (nWByte < nAdBufptr - nWTotalByte) return FALSE;
//     nWTotalByte = nWByte + nWTotalByte;
//  }
    return TRUE;
}



//////////////////////////////////////////////////////////////////////////////
//	ConvertAdpcm_to_Wave8m()
//
// Copyright 1992, ZyXEL Communications Corporation
// Windows Version Modified By Valen Liao   Apr22 "95
//
int ConvertAdpcm_to_Wave8m(
char * npszAd,
char * npszWave,
int nSampRate)
{
   char szAdpcmHeader[16];

   BYTE cbAdpcm;
   WORD wVocData1,wVocData2,wVocData3,wVocData4;
   WORD wVocData5,wVocData6,wVocData7,wVocData8,wVocData9;
   int bfEof = FALSE;

   ZVAPIFILECONV * npzfAdV;

   ZVAPIFILECONV zfTempAdV;


   ZVRIFF zvWave;
   ZVRIFF zvAdpcmVoc;
   ZVINTERPOLATIONPARAM zvInterpPm;
   ZVINTERPOLATIONRESULT zvInterpResult;

   char * npAdBuf;
   WORD wRByte;
   char * npWaveBuf;
   int nWaveBufptr;

   FILE *fAd, *fWave;

   if(NULL == (fWave = fopen(npszWave, "wb"))) return -1;
   if(NULL == (fAd = fopen(npszAd, "r+b"))) return -1;

   // Check input File for // ZyXEL Header & Adpcm compress bit Flag
   fread(szAdpcmHeader, sizeof(char), 16, fAd);
   if (0 != strcmp(szAdpcmHeader, "ZyXEL")) { fclose(fAd); return -1; }


   // Allcate Memory for Convert Struct Adpcm to VOC
   npzfAdV = (ZVAPIFILECONV * ) calloc(sizeof(ZVAPIFILECONV), 1);
   // Allcate Memory for Convert Buffer for reading Adpcm File
   npAdBuf = calloc(CV_SIZE, 1);
   // Allcate Memory for Convert Buffer for writing 8 bits nomo Wave File
   npWaveBuf = calloc(CV_SIZE * 8, 1);

   // Initial parameters Adpcm to Voc & Check if Input file is not
   // Adpcm File of ZyXEL
   npzfAdV->cbCompBit = szAdpcmHeader[10];
   if (npzfAdV->cbCompBit != 1 && // Adpcm 2 bit
       npzfAdV->cbCompBit != 2 ) // Adpcm 3 bit
   {
      fclose(fAd);
      free(npzfAdV);
      free(npAdBuf);
      free(npWaveBuf);
      return -1;
   }

   npzfAdV->nRdata = 0;
   npzfAdV->cbRcnt = 8;
   npzfAdV->nDelta = 5;
   npzfAdV->nEstMax = 0;
   if (npzfAdV->cbCompBit == 1)
      memcpy((VOID *)npzfAdV->nMaxTbl, (VOID *)_nMaxTbl2Bit, 4*sizeof(int));
   else
      memcpy((VOID *)npzfAdV->nMaxTbl, (VOID *)_nMaxTbl3Bit, 4*sizeof(int));

   // Initialize RIFF Header for writing File & Adpcm Convert to VOC Header
   InitialRIFFHeader((ZVRIFF *)&zvWave, nSampRate);
   InitialRIFFHeader((ZVRIFF *)&zvAdpcmVoc, 9600);

   // Write RIFF header for converted header
   fwrite(&zvWave, sizeof(char), 44, fWave);

   // Initialize InterPolation Parameter
   InitialInterPolation((ZVINTERPOLATIONPARAM *)&zvInterpPm,
			(ZVRIFF *)&zvAdpcmVoc,
			(ZVRIFF *)&zvWave);

   // Start Reading Adpcm File data & Conerting to RIFF Wave File
   while(!bfEof)
   {
       int i;

       if (0 == (wRByte = fread(npAdBuf, sizeof(char), CV_SIZE, fAd)))
       {
	  bfEof = TRUE; break;
       }


       nWaveBufptr = 0;
       for (i=0; i<(int)wRByte; i)
       {
	  if (npzfAdV->cbCompBit == 1)
	  {
	     cbAdpcm = npAdBuf[i];
	     wVocData1 = ToVoc(((cbAdpcm & 0xC0) >> 6), (LPSTR)npzfAdV);
	     wVocData2 = ToVoc(((cbAdpcm & 0x30) >> 4), (LPSTR)npzfAdV);
	     wVocData3 = ToVoc(((cbAdpcm & 0x0C) >> 2), (LPSTR)npzfAdV);
	     wVocData4 = ToVoc((cbAdpcm & 0x03), (LPSTR)npzfAdV);

	     memcpy((VOID *)&zfTempAdV, (VOID *)npzfAdV, sizeof(ZVAPIFILECONV));
	     if ((i+1) < (int)wRByte)
		wVocData5 = ToVoc(((npAdBuf[i+1] & 0xC0)>>6), (LPSTR)npzfAdV);
	     else
	     {
		if (0 == fread(&cbAdpcm, sizeof(char), 1, fAd))
		   bfEof = TRUE;
		else
		{
		   _lseekf(fAd, -1L, SEEK_CUR);
		   wVocData5 = ToVoc(((cbAdpcm & 0xC0)>>6), (LPSTR)npzfAdV);
		}
	     }

	     memcpy((VOID *)npzfAdV, (VOID *)&zfTempAdV, sizeof(ZVAPIFILECONV));

	     if (Interpolation(LOBYTE(wVocData1), LOBYTE(wVocData2),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData2), LOBYTE(wVocData3),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData3), LOBYTE(wVocData4),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (bfEof) break;
	     if (Interpolation(LOBYTE(wVocData4), LOBYTE(wVocData5),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     i++;
	  }
	  else
	  {
	     cbAdpcm = npAdBuf[i++];
	     // Data Bit : XXX- ----
	     wVocData1 = ToVoc(((cbAdpcm & 0xE0) >> 5), (LPSTR)npzfAdV);

	     // Data Bit : ---X XX--
	     wVocData2 = ToVoc(((cbAdpcm & 0x1C) >> 2), (LPSTR)npzfAdV);
	     npzfAdV->nPack = (cbAdpcm & 0x03) << 1;
	     cbAdpcm = npAdBuf[i++];

	     // Data Bit : ---- --XX X--- ----
	     wVocData3 = ToVoc(npzfAdV->nPack | ((cbAdpcm & 0x80) >> 7),
			       (LPSTR)npzfAdV);

	     // Data Bit : -XXX ----
	     wVocData4 = ToVoc(((cbAdpcm & 0x70) >> 4), (LPSTR)npzfAdV);

	     // Data Bit : ---- XXX-
	     wVocData5 = ToVoc(((cbAdpcm & 0x0E) >> 1), (LPSTR)npzfAdV);
	     npzfAdV->nPack = (cbAdpcm & 0x01) << 2;
	     cbAdpcm = npAdBuf[i++];

	     // Data Bit : ---- ---X XX-- ----
	     wVocData6 = ToVoc((npzfAdV->nPack | ((cbAdpcm & 0xC0) >> 6)),
			       (LPSTR)npzfAdV);

	     // Data Bit : --XX X---
	     wVocData7 = ToVoc(((cbAdpcm & 0x38) >> 3), (LPSTR)npzfAdV);

	     // Data Bit : ---- -XXX
	     wVocData8 = ToVoc((cbAdpcm & 0x07), (LPSTR)npzfAdV);

	     memcpy((VOID *)&zfTempAdV, (VOID *)npzfAdV, sizeof(ZVAPIFILECONV));
	     if ((i) < (int)wRByte)
		wVocData9 = ToVoc(((npAdBuf[i] & 0xE0)>>5), (LPSTR)npzfAdV);
	     else
	     {
		if (0 == fread(&cbAdpcm, sizeof(char), 1, fAd))
		   bfEof = TRUE;
		else
		{
		   LONG lbyte;
		   lbyte = _lseekf(fAd, 0, SEEK_CUR);
		   _lseekf(fAd, lbyte-1, SEEK_SET);
		   wVocData9 = ToVoc(((cbAdpcm & 0xE0)>>5), (LPSTR)npzfAdV);
		}
	     }
	     memcpy((VOID *)npzfAdV, (VOID *)&zfTempAdV, sizeof(ZVAPIFILECONV));

	     if (Interpolation(LOBYTE(wVocData1), LOBYTE(wVocData2),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData2), LOBYTE(wVocData3),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData3), LOBYTE(wVocData4),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData4), LOBYTE(wVocData5),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData5), LOBYTE(wVocData6),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData6), LOBYTE(wVocData7),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	     if (Interpolation(LOBYTE(wVocData7), LOBYTE(wVocData8),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }

	     if (bfEof) break;
	     if (Interpolation(LOBYTE(wVocData8), LOBYTE(wVocData9),
			       (ZVINTERPOLATIONPARAM *)&zvInterpPm,
			       (ZVINTERPOLATIONRESULT *)&zvInterpResult))
	     {
		memcpy((VOID *)(npWaveBuf + nWaveBufptr),
		       (VOID *)zvInterpResult.szPCMData,
		       zvInterpResult.nPCMDataPtr);
		nWaveBufptr += zvInterpResult.nPCMDataPtr;
	     }
	  }
       }// end for()


       if (!WriteConvertFile(fWave, npWaveBuf, nWaveBufptr))
	  break;
       nWaveBufptr = 0;
   }

   zvWave.dwFileLen = _lseekf(fWave, 0, SEEK_END) - 8L;
   zvWave.dwDataLen = _lseekf(fWave, 0, SEEK_END) - 44L;
   _lseekf(fWave, 0, SEEK_SET);
   fwrite(&zvWave, sizeof(char), 44, fWave);

   fclose(fAd);
   fclose(fWave);

   // Free ZVAPIFILECONV struct & Disk R/W buffer
   free(npzfAdV);
   free(npAdBuf);
   free(npWaveBuf);

   return 0;
}



//////////////////////////////////////////////////////////////////////////////
//
//
//
VOID InitialRIFFHeader(
ZVRIFF *npZvRiff,
int nSampRate)
{
   memcpy((VOID *)npZvRiff->szRiff, (VOID *)"RIFF", 4);
   npZvRiff->dwFileLen = 32;
   memcpy((VOID *)npZvRiff->szWave, (VOID *)"WAVEfmt ", 8);

   npZvRiff->dwFmtCategory = 0x10;
   npZvRiff->wFmtTag = 1; // PCM
   npZvRiff->wChannel = 1; // MONO
   npZvRiff->dwSampPerSec = nSampRate; // 11.025kHz, 22.05kHz, 44.1kHz
   npZvRiff->dwAvgBytePerSec = nSampRate; //

   npZvRiff->wBlkAlign = 1; // 8 bits, mono Block alignment
   npZvRiff->wBitsPerSamp = 8;
   memcpy((VOID *)npZvRiff->szData, (VOID *)"data", 4);
   npZvRiff->dwDataLen = 0;
   return;
}


//////////////////////////////////////////////////////////////////////////////
//	InitialPolation()
//
// Copyright 1992, ZyXEL Communications Corporation
// Windows Version Modified by Valen Liao ,Apr22 "95
//
VOID InitialInterPolation(
ZVINTERPOLATIONPARAM *npzvInterpPm,
ZVRIFF *npzvAdpcm,
ZVRIFF *npzvWave)
{
   npzvInterpPm->Ti = (float)(1.0 / (float)npzvAdpcm->dwSampPerSec); // in
   npzvInterpPm->To = (float)(1.0 / (float)npzvWave->dwSampPerSec);  // out
   npzvInterpPm->Tnow = npzvInterpPm->To;
   return;
}


//////////////////////////////////////////////////////////////////////////////
//	InitialDecimation()
//
// Copyright 1992, ZyXEL Communications Corporation
// Windows Version Modified by Valen Liao, Apr22 "95
//
VOID InitialDecimation(
ZVINTERPOLATIONPARAM *npzvInterpPm,
ZVRIFF *npzvAdpcm,
ZVRIFF *npzvWave)
{
   npzvInterpPm->To = (float)(1 / (float)npzvAdpcm->dwSampPerSec); // Out
   npzvInterpPm->Ti = (float)(1 / (float)npzvWave->dwSampPerSec);  // In
   npzvInterpPm->Tnow = npzvInterpPm->Ti;
   return;
}


//////////////////////////////////////////////////////////////////////////////
//	InterPolation()
//
//	Resample Interpolation the 8 bit PCM code, The Input data sample rate
//	must be less then the output data sample rate
//
//
// Copyright 1992, ZyXEL Communications Corporation
// Windows Version Modified by Valen Liao, Apr22 "95
//
int Interpolation(
BYTE cbSpn_1,
BYTE cbSpn_2,
ZVINTERPOLATIONPARAM * npzvInterpPm,
ZVINTERPOLATIONRESULT * npzvInterpResult)
{
   BYTE cbOutData;
   int bfPack = FALSE;

   memset((VOID *)npzvInterpResult->szPCMData, 0, 100);
   npzvInterpResult->nPCMDataPtr = 0;

   while (npzvInterpPm->Tnow < npzvInterpPm->Ti)
   {
      cbOutData =
//    cbSpn_1 + (BYTE)((npzvInterpPm->Tnow/npzvInterpPm->Ti)*(cbSpn_2-cbSpn_1));
      cbSpn_1 + LOBYTE((npzvInterpPm->Tnow/npzvInterpPm->Ti)*(cbSpn_2-cbSpn_1));
      npzvInterpResult->szPCMData[npzvInterpResult->nPCMDataPtr++] = cbOutData;
      npzvInterpPm->Tnow += npzvInterpPm->To;
      bfPack = TRUE;
   }
   npzvInterpPm->Tnow -= npzvInterpPm->Ti;
   return bfPack;
}




//////////////////////////////////////////////////////////////////////////////
//	Decimation()
//
//	Resample Decimation the 8 bit PCM code, The Input data sample rate
//	must be greater then the output data sample rate
//
//
// Copyright 1992, ZyXEL Communications Corporation
// Windows Version Modified by Valen Liao, Apr22 "95
//
int Decimation(
BYTE cbSpn_1,
BYTE cbSpn_2,
ZVDECIMATIONPARAM *npzvDecimatPm,
ZVDECIMATIONRESULT *npzvDecimatResult)
{
   BYTE cbOutData;
   float fDelta;

   memset((VOID *)npzvDecimatResult->szPCMData, 0, 100);
   npzvDecimatResult->nPCMDataPtr = 0;
   if (npzvDecimatPm->Tnow <= npzvDecimatPm->To)
   {
      npzvDecimatPm->Tnow += npzvDecimatPm->Ti;
      return FALSE;
   }

   fDelta = npzvDecimatPm->To + npzvDecimatPm->Ti - npzvDecimatPm->Tnow;
   npzvDecimatPm->Tnow -= npzvDecimatPm->To;

   cbOutData = cbSpn_1 +
	       (BYTE)((fDelta/npzvDecimatPm->Ti)*(float)(cbSpn_2 - cbSpn_1));

   npzvDecimatResult->szPCMData[npzvDecimatResult->nPCMDataPtr++] = cbOutData;
   return TRUE;
}






//////////////////////////////////////////////////////////////////////////////
//	Wave8m_to_ConvertAdpcm()
//
// Copyright 1992, ZyXEL Communications Corporation
// Modified By Valen Liao, Apr22 "95
//
int ConvertWave8m_to_Adpcm(
char *npszWave,
char *npszAd,
int nDf)
{
   char szAdpcmHeader[16] = {'Z','y','X','E','L',2,0,0,3,0,0,0,0,0,0,0};

   int bfEof = FALSE;
   BYTE cbDf = LOBYTE(nDf);

   ZVAPIFILECONV *npzfVAd;

   ZVRIFF zvWave;
   ZVRIFF zvVocAdpcm;

   ZVDECIMATIONPARAM zvDecimatPm;
   ZVDECIMATIONRESULT zvDecimatResult;

   ZVAPITOADPCMRESULT zfadpcm;

   char * npAdBuf;
   int nAdBufptr;
   char * npWaveBuf;
   WORD wRByte;

   FILE *fAd, *fWave;

   // Check Adpcm 2 / 3 bit Decompress Format
   if (cbDf != DF_ADPCM2BIT && cbDf != DF_ADPCM3BIT)
      return -1;

   if(NULL == (fAd = fopen(npszAd, "wb"))) return -1;
   if(NULL == (fWave = fopen(npszWave, "r+b"))) return -1;

   // Check input File for // ZyXEL Header & Adpcm compress bit Flag
   if (!checkWave8mHeader(fWave, (ZVRIFF *)&zvWave))
   {
      fclose(fWave); return -1;
   }

   // Allcate Memory for Convert Struct VOC to adpcm
   npzfVAd = (ZVAPIFILECONV * ) calloc(sizeof(ZVAPIFILECONV), 1);
   // Allcate Memory for Convert Buffer for Writeing Adpcm File
   npAdBuf = calloc(8*CV_SIZE, 1);
   // Allcate Memory for Convert Buffer for writing 8 bits nomo Wave File
   npWaveBuf = calloc(CV_SIZE, 1);

   // Initial parameters Adpcm to Voc & Check if Input file is not
   // Adpcm File of ZyXEL
   npzfVAd->cbCompBit = (BYTE)cbDf-1;
   szAdpcmHeader[10] = (BYTE)cbDf-1;
   if (npzfVAd->cbCompBit != 1 && // Adpcm 2 bit
       npzfVAd->cbCompBit != 2 ) // Adpcm 3 bit
   {
      fclose(fWave);
      free(npzfVAd);
      free(npAdBuf);
      free(npWaveBuf);
      return -1;
   }

   npzfVAd->nRdata = 0;
   npzfVAd->cbRcnt = 8;
   npzfVAd->nDelta = 5;
   npzfVAd->nEstMax = 0;

   if (npzfVAd->cbCompBit == 1)
      memcpy((VOID *)npzfVAd->nMaxTbl, (VOID *)_nMaxTbl2Bit, 4*sizeof(int));
   else
      memcpy((VOID *)npzfVAd->nMaxTbl, (VOID *)_nMaxTbl3Bit, 4*sizeof(int));

   // Initialize Adpcm Header of ZyXEL
   szAdpcmHeader[10] = npzfVAd->cbCompBit;
   fwrite(szAdpcmHeader, sizeof(char), 16, fAd);

   // Initialize Adpcm Riff data struct
   InitialRIFFHeader((ZVRIFF *)&zvVocAdpcm, 9600);

   // Initialize Decimation Parameter
   InitialDecimation((ZVDECIMATIONPARAM *)&zvDecimatPm,
		     (ZVRIFF *)&zvVocAdpcm,
		     (ZVRIFF *)&zvWave);

   // Start Reading Wave(RIFF) File data & Conerting to Adpcm Wave File
   while(!bfEof)
   {
       int i;

       if (0 == (wRByte = fread(npWaveBuf, sizeof(char), CV_SIZE, fWave)))
       {
	  bfEof = TRUE; break;
       }

       nAdBufptr = 0;
       for (i=0; i<(int)wRByte; i++)
       {
	    BYTE cbNextWave;
	    if((i+1) < (int)wRByte)
	       cbNextWave = npWaveBuf[i+1];
	    else
	    {
	       if (0 == fread(&cbNextWave, sizeof(char), 1, fWave))
	       {
		  bfEof = TRUE; break;
	       }
	       else
		  _lseekf(fWave, -1L, SEEK_CUR);
	    }
	    if (Decimation(npWaveBuf[i],
			   cbNextWave,
			   (ZVDECIMATIONPARAM *)&zvDecimatPm,
			   (ZVDECIMATIONRESULT *)&zvDecimatResult))
	    {
	       int j;
	       for(j=0; j<zvDecimatResult.nPCMDataPtr; j++)
	       {
		  if (ToAdpcm(MAKEWORD(zvDecimatResult.szPCMData[j], 0),
			      (LPSTR)npzfVAd,
			      (LPSTR)&zfadpcm))
		  {
		     memcpy((VOID *)(npAdBuf + nAdBufptr),
			    (VOID *)zfadpcm.szAdpcmData,
			    zfadpcm.nDataByte);
		     nAdBufptr += zfadpcm.nDataByte;
		  }

	       } // end for for(j=0; j<zvDecimatResult.nPCMDataPtr, j++)
	       i--;
	    } // end of if Decimation()
       }// end for()

       if (!WriteConvertFile(fAd, npAdBuf, nAdBufptr)) break;
       nAdBufptr = 0;
   } // end of while()

   fclose(fWave);
   fclose(fAd);

   // Free ZVAPIFILECONV struct & Disk R/W Buffer
   free(npzfVAd);
   free(npAdBuf);
   free(npWaveBuf);

   return 0;
}




//////////////////////////////////////////////////////////////////////////////
//
//
//
int checkWave8mHeader(
FILE *fWave,
ZVRIFF *npzvWave)
{
   // check Argument
   DWORD dwFmtCategoryLen;

   // Initialize the Wave File RIFF structure
   memset((VOID *)npzvWave, 0, 44);

   // Read "RIFF"
   fread(npzvWave->szRiff, sizeof(char), 4, fWave);
   if(strcmp(npzvWave->szRiff, "RIFF") != 0) return FALSE;

   // Read file Length
   fread(&npzvWave->dwFileLen, sizeof(char), 4, fWave);
   if (npzvWave->dwFileLen == 0) return FALSE;

   // Read "WAVEfmt " header
   fread(npzvWave->szWave, sizeof(char), 8, fWave);
   if(strcmp(npzvWave->szWave, "WAVEfmt ") != 0) return FALSE;

   // Read Wave Information tag Length
   fread(&dwFmtCategoryLen, sizeof(char), 4, fWave);
   if (dwFmtCategoryLen < 16) return FALSE;
   npzvWave->dwFmtCategory = dwFmtCategoryLen;

   // Read PCM FmtTag
   fread(&npzvWave->wFmtTag, sizeof(char), 2, fWave);
   if (npzvWave->wFmtTag != 1) return FALSE;	// Not PCM Data

   // Read Channel Number
   fread(&npzvWave->wChannel, sizeof(char), 2, fWave);
   if (npzvWave->wChannel != 1) return FALSE;	// Not mono file

   // Read Sample per Second
   fread(&npzvWave->dwSampPerSec, sizeof(char), 4, fWave);
   if (npzvWave->dwSampPerSec < 9600) return FALSE;

   // Read Aveage Byte per Second
   fread(&npzvWave->dwAvgBytePerSec, sizeof(char), 4, fWave);
   if (npzvWave->dwAvgBytePerSec < 9600) return FALSE;

   // Read BlockAlignment
   fread(&npzvWave->wBlkAlign, sizeof(char), 2, fWave);
   if (npzvWave->wBlkAlign != 1)  return FALSE; //Not 8bits mono file

   // Read Bits Per Sample
   fread(&npzvWave->wBitsPerSamp, sizeof(char), 2, fWave);
   if (npzvWave->wBitsPerSamp != 8) return FALSE;	//Not 8bits mono file

   if (dwFmtCategoryLen > 16)
   {
	dwFmtCategoryLen -= 16;
	_lseekf(fWave, (LONG)dwFmtCategoryLen, SEEK_CUR);
   }

   // Read "data"
   fread(npzvWave->szData, sizeof(char), 4, fWave);
   if (strcmp(npzvWave->szData, "data") != 0) return FALSE;

   fread(&npzvWave->dwDataLen, sizeof(char), 4, fWave);
   if (npzvWave->dwDataLen == 0) return FALSE;

   return TRUE;
}


void cvAdpcm2Wave(
char * npszAdpcmFn,
char * npszWaveFn,
int * pnSampRate)
{
   printf("\n\r");
   printf("Enter source file name (ADPCM Zfax file format only) : ");
   scanf("%s", npszAdpcmFn);
   printf("Enter output file name (prepared for wave file output) : ");
   scanf("%s", npszWaveFn);
   printf("Enter output sampeling rate, in Hz : ");
   scanf("%d", pnSampRate);
   printf("\n\r");

   if (0==ConvertAdpcm_to_Wave8m(npszAdpcmFn, npszWaveFn, *pnSampRate))
      printf("Adpsm to Wave format Convert ending.\n\r");
   else
      printf("There are some errors ! check it.\n\r");
   return;
}


void cvWave2Adpcm(
char * npszWaveFn,
char * npszAdpcmFn,
int * pnDf)
{
   printf("\n\r");
   printf("Enter source file name (Wave 8bits mono file format only) : ");
   scanf("%s", npszWaveFn);
   printf("Enter output file name (prepared for Zfax voice file output) : ");
   scanf("%s", npszAdpcmFn);
   printf("Enter output file format (0 for ADPCM 2bit, 1 for ADPCM 3bit) : ");
   scanf("%d", pnDf);
   printf("\n\r");
   *pnDf += DF_ADPCM2BIT;

   if (0==ConvertWave8m_to_Adpcm(npszWaveFn, npszAdpcmFn, *pnDf))
      printf("Wave to Adpsm format Convert ending.\n\r");
   else
      printf("There are some errors ! check it.\n\r");
   return;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Main Procedure
//
void main()
{
   char szAdpcmFn[256];
   char szWaveFn[256];
   int nChoice;
   int nSampRate;
   int nDF;

   printf("\n\r");
   printf("Enter '1' for converting ADPCM to Wave, '2' for converting WAVE to ADPCM : ");
   scanf("%d", &nChoice);
   if (nChoice == 1)
       cvAdpcm2Wave(szAdpcmFn, szWaveFn, &nSampRate);
   else
   if (nChoice == 2)
       cvWave2Adpcm(szWaveFn, szAdpcmFn, &nDF);
   else
       printf("Error choice !\n\r");
   return;
}
